@@ -1,3 +1,54 @@
+// Signed Labeling -----------------------------------------------------------------------------------------------------
+
+/// Labels for signed integer primitives.
+#[derive(Debug, PartialEq, Eq, Ord, PartialOrd, Copy, Clone)]
+pub enum SmallSignedLabel {
+    /// A label for `isize` types.
+    ISIZE,
+
+    /// A label for `i8` types.
+    I8,
+
+    /// A label for `i16` types.
+    I16,
+
+    /// A label for `i32` types.
+    I32,
+
+    /// A label for `i64` types.
+    I64,
+
+    /// A label for `i128` types.
+    I128,
+}
+
+// TODO: return ISIZE based on host width?
+impl SmallSignedLabel {
+    /// Maps input `isize` to label for smallest integer primitive capable of representing it
+    /// (e.g. `new(-100)` -> `SmallSignedLabel::I8`).
+    /// At present, this function does not return the `ISIZE` variant (never needed?).
+    pub const fn new(num: isize) -> Self {
+        if (core::i8::MIN as i128 <= (num as i128)) && ((num as i128) <= (core::i8::MAX as i128)) {
+            SmallSignedLabel::I8
+        } else if (core::i16::MIN as i128 <= (num as i128))
+            && ((num as i128) <= (core::i16::MAX as i128))
+        {
+            SmallSignedLabel::I16
+        } else if (core::i32::MIN as i128 <= (num as i128))
+            && ((num as i128) <= (core::i32::MAX as i128))
+        {
+            SmallSignedLabel::I32
+        } else if (core::i64::MIN as i128 <= (num as i128))
+            && ((num as i128) <= (core::i64::MAX as i128))
+        {
+            SmallSignedLabel::I64
+        } else {
+            // (core::i128::MIN as i128 <= (num as i128)) && ((num as i128) <= (core::i128::MAX as i128))
+            SmallSignedLabel::I128
+        }
+    }
+}
+
 // Signed Normalization ------------------------------------------------------------------------------------------------
 
 // TODO: make this const once stabilized: https://github.com/rust-lang/rust/issues/67792
@@ -5,16 +56,31 @@
 
 /// Convenience trait for signed normalization (e.g. to/from `isize`).
 pub trait SmallSigned {
-    /// Get value of small signed as host register-width signed (e.g. `isize`)
+    /// **Upcast:** Get value of small signed as host register-width signed (e.g. `isize`)
     fn isize(&self) -> isize;
 
-    /// Convert input `isize` into a primitive implementing the `SmallSigned` trait.
+    /// **Downcast:** Convert input `isize` into a primitive implementing the `SmallSigned` trait.
     /// Panics if `isize` exceeds min/max for returned signed primitive.
     /// `core::convert::From` not used b/c `SmallSigned` is not generic by design,
     /// implemented only for (`i8`, `i16`, `i32`, `i64`, `i128`) and only up to host integer width.
+    ///
+    /// ### Note
+    ///
+    /// Unlike others, this API has a tiny (1 comparison/branch) runtime cost.
+    /// The `check` in `checked_from` is an `assert` to prevent loss of precision.
     fn checked_from(num: isize) -> Self;
 }
 
+impl SmallSigned for isize {
+    fn isize(&self) -> isize {
+        *self
+    }
+
+    fn checked_from(num: isize) -> isize {
+        num
+    }
+}
+
 impl SmallSigned for i8 {
     fn isize(&self) -> isize {
         *self as isize
@@ -83,9 +149,9 @@ impl SmallSigned for i128 {
     }
 }
 
-// Compile-time Bound Mapping ------------------------------------------------------------------------------------------
+// Compile-time Type Mapping -------------------------------------------------------------------------------------------
 
-/// Return smallest signed type capable of representing input value (positive, i.e. maximum, or negative, i.e. minimum)
+/// Return smallest signed type capable of representing input value (positive, i.e. maximum, or negative, i.e. minimum).
 ///
 /// # Example
 ///
@@ -133,6 +199,7 @@ macro_rules! small_signed {
     };
 }
 
+#[doc(hidden)] // API user should never have to be aware this exists.
 /// Helper trait for signed type mapping. Internal use only.
 pub trait ShrinkSigned<
     const FITS_I8: bool,
@@ -166,12 +233,35 @@ impl ShrinkSigned<false, false, false, false, true> for () {
     type SmallSigned = i128;
 }
 
+// Compile-time Label Mapping ------------------------------------------------------------------------------------------
+
+/// Return a label corresponding to the smallest type capable of representing input value
+/// (positive, i.e. maximum, or negative, i.e. minimum).
+///
+/// # Example
+///
+/// ```
+/// use smallnum::{small_signed_label, SmallSignedLabel};
+///
+/// let i8_label = small_signed_label!(100);
+/// assert_eq!(i8_label, SmallSignedLabel::I8);
+///
+/// let i16_label = small_signed_label!(-500);
+/// assert_eq!(i16_label, SmallSignedLabel::I16);
+/// ```
+#[macro_export]
+macro_rules! small_signed_label {
+    ( $max:expr $(,)? ) => {
+        SmallSignedLabel::new($max)
+    };
+}
+
 // Test ----------------------------------------------------------------------------------------------------------------
 
 #[cfg(test)]
 mod tests {
 
-    use crate::SmallSigned;
+    use crate::{SmallSigned, SmallSignedLabel};
     use core::mem::size_of;
     use static_assertions::assert_type_eq_all;
 
@@ -316,4 +406,55 @@ mod tests {
             );
         }
     }
+
+    #[test]
+    fn signed_label_macro() {
+        // Label mapping -----------------------------------------------------------------------------------------------
+
+        let max_label = small_signed_label!(MAX_VAL_SIGNED);
+        let i8_label_pos = small_signed_label!(100);
+        let i16_label_pos = small_signed_label!(500);
+        let i32_label_pos = small_signed_label!(50_000);
+
+        #[cfg(any(target_pointer_width = "64", target_pointer_width = "128"))]
+        let i64_label_pos = small_signed_label!(2_200_000_000);
+
+        #[cfg(target_pointer_width = "128")]
+        let i128_label_pos = small_signed_label!(9_300_000_000_000_000_000);
+
+        let min_label = small_signed_label!(MIN_VAL_SIGNED);
+        let i8_label_neg = small_signed_label!(-100);
+        let i16_label_neg = small_signed_label!(-500);
+        let i32_label_neg = small_signed_label!(-50_000);
+
+        #[cfg(any(target_pointer_width = "64", target_pointer_width = "128"))]
+        let i64_label_neg = small_signed_label!(-2_200_000_000);
+
+        #[cfg(target_pointer_width = "128")]
+        let i128_label_neg = small_signed_label!(-9_300_000_000_000_000_000);
+
+        // Label Check ---------------------------------------------------------------------------------------------------
+
+        assert_eq!(max_label, SmallSignedLabel::I16);
+        assert_eq!(i8_label_pos, SmallSignedLabel::I8);
+        assert_eq!(i16_label_pos, SmallSignedLabel::I16);
+        assert_eq!(i32_label_pos, SmallSignedLabel::I32);
+
+        #[cfg(any(target_pointer_width = "64", target_pointer_width = "128"))]
+        assert_eq!(i64_label_pos, SmallSignedLabel::I64);
+
+        #[cfg(target_pointer_width = "128")]
+        assert_eq!(i128_label_pos, SmallSignedLabel::I128);
+
+        assert_eq!(min_label, SmallSignedLabel::I16);
+        assert_eq!(i8_label_neg, SmallSignedLabel::I8);
+        assert_eq!(i16_label_neg, SmallSignedLabel::I16);
+        assert_eq!(i32_label_neg, SmallSignedLabel::I32);
+
+        #[cfg(any(target_pointer_width = "64", target_pointer_width = "128"))]
+        assert_eq!(i64_label_neg, SmallSignedLabel::I64);
+
+        #[cfg(target_pointer_width = "128")]
+        assert_eq!(i128_label_neg, SmallSignedLabel::I128);
+    }
 }
